﻿/*
(c) 2009-2019, W.Happe, www.ophthalmostar.de
Timer-Periode kann nicht länger als 4294967295 Millisekunden sein (49.7 days).

Letzte Änderungen:
22.03.14: TrayTip wenn INI-Datei angelegt werden muss (erstmaliger Start)
20.01.14: "DiffTime > 1" statt "DiffTime > 0" um mehrfache Starts zu verhindern
31.01.14: "DiffTime"-Berechnung abhängig von "ExecNum := AlarmRerun%AIndex%"
09.02.14: "ActionTime"-Vergleich um Mehrfachstarts zu verhindern
*/
#NoEnv
#NoTrayIcon
#SingleInstance, Force
SendMode, Input  ; Recommended for new scripts due to its superior speed and reliability
If InStr(FileExist(A_ScriptDir "\" A_ComputerName), "D")
  SetWorkingDir, %A_ScriptDir%\%A_ComputerName%
Else
  SetWorkingDir, %A_ScriptDir%
ListLines, Off ; "ListLines Off" may improve performance by a few percent
SetBatchLines, -1 ; run at maximum speed; default is 10ms
Critical ; turning on Critical also puts SetBatchLines -1
;Process, Priority,, High ; PID-or-Name: if blank the priority of the script itself
ScriptName = AutoHotkeyClock
ScriptVersion = 1.0 ; die letzten beiden Ziffen werden automatisch erhöht
ScriptBuild = 1.1 ; die letzten beiden Ziffen werden automatisch erhöht
ScriptDate = 22.9.2019
webSite = https://www.ophthalmostar.de/freeware/#autohotkey
ColorMH := "000066" ; Hex of Minute and Hour Hand
Color_S := "660000" ; Hex of Second Hand
SplitPath, A_ScriptFullPath,,,, ScriptNoExt
; ------------------------------------------------------------------------------
IniPath = %A_WorkingDir%\%ScriptNoExt%.ini
; ------------------------------------------------------------------------------
OnExit, EXITPROG
; ------------------------------------------------------------------------------
; Localisation
; ------------------------------------------------------------------------------
Language := SubStr(A_Language, 3, 2) = 07 ? "german" : "english" ; 0407, 0807, 0c07, 1007, 1407
If Language = english
  {
    lng_Alarms = Alarms
    lng_EggTimer = Set Egg Timer
    lng_Minutes = minutes
    lng_Options = Options
    lng_HideClock = Hide Clock
    lng_About = About
    lng_ClickThrough = Click Through
    lng_Exit = Exit
    lng_Gdiplus = Gdiplus error.`n%ScriptName% exits.
    lng_Download = Downloading picture. Please wait...
    lng_NoPicture = %ScriptNoExt%.png» was not found and could not be downloaded.`nFor that reason the program exits.
    lng_New = New
    lng_Edit = Edit
    lng_Duplicate = Duplicate
    lng_Delete = Delete
    lng_DoItNow = Execute now
    lng_Properties = Properties
    lng_Del = Del
  }
Else
  {
    lng_Alarms = Alarme
    lng_EggTimer = Eieruhr stellen
    lng_Minutes = Minuten
    lng_Options = Einstellungen
    lng_HideClock = Uhr ausblenden
    lng_About = Über
    lng_ClickThrough = Hindurchklickbar
    lng_Exit = Beenden
    lng_Gdiplus = Gdiplus konnte nicht gestartet werden.`n%ScriptName% wird beendet.
    lng_Download = Bild wird herutergeladen. Bitte warten...
    lng_NoPicture = %ScriptNoExt%.png» wurde nicht gefunden und konnte auch`nnicht heruntergeladen werden.`nDas Programm kann daher nicht ausgeführt werden.
    lng_New = Hinzufügen
    lng_Edit = Bearbeiten
    lng_Duplicate = Duplizieren
    lng_Delete = Löschen
    lng_DoItNow = Testen
    lng_Properties = Eigenschaft
    lng_Del = Entf
  }
; ------------------------------------------------------------------------------
; UserInit
; ------------------------------------------------------------------------------
Loop  ; Fortsetzung erst wenn userinit.exe beendet ist
  { ; Uhr wurde manchmal falsch positioniert
  Process, Exist, UserInit.exe
  IfEqual, ErrorLevel, 0, Break
  Sleep, 100
  }
; ------------------------------------------------------------------------------
; Tray Menu
; ------------------------------------------------------------------------------


MenuItemShowHide = &Uhr anzeigen
Menu, Tray, NoStandard
arrayEggMenu := [90, 60, 45, 30, 20, 15, 10, 5, 3, 2]  ; hier noch ohne "Minuten"
For index, value in arrayEggMenu
{
  arrayEggMenu[index] := value . A_Space . lng_Minutes
  Menu, SubMenu, Add, % arrayEggMenu[index], EggTimer
}
Menu, Tray, Add, &%lng_About%..., ABOUT
Menu, Tray, Add, &Datum und Uhrzeit, DateAndTime
Menu, Tray, Add
Menu, Tray, Add, &%lng_Alarms%..., ALARMS
Menu, Tray, Add, &%lng_EggTimer%, :SubMenu
Menu, Tray, Add
Menu, Tray, Add, &%lng_Options%..., OPTIONEN
Menu, Tray, Add, %MenuItemShowHide%, HideShowGui
Menu, Tray, Add, %lng_ClickThrough%, NotClickable
Menu, Tray, Add
If (!A_IsCompiled)
  {
  Menu, AutoHotkey, Standard
  Menu, Tray, Add, AutoHotkey, :AutoHotkey
;  Menu, Tray, Add
  }
Menu, Tray, Add, &%lng_Exit%, PROGEXIT
If A_OSVersion in WIN_VISTA,WIN_7
  Menu, Tray, Icon, shell32.dll, 266, 1
Else
  Menu, Tray, Icon, shell32.dll, 21, 1 ; wmploc.dll, 18
Menu, Tray, Default, %MenuItemShowHide% ; &Uhr anzeigen
Menu, Tray, Click, 1
Menu, Tray, Check, %MenuItemShowHide%

; ------------------------------------------------------------------------------
; INI-File
; ------------------------------------------------------------------------------
IfNotExist, %IniPath%
  {
  Menu, Tray, Icon
  TrayTip, %ScriptName%, Klicken Sie hier mit der rechten Maustaste`,`num AHK-Clock-Einstellungen zu verändern., 30, 16
  FileAppend,
    (
[General]
    ), %IniPath%
  }
SysGet, WorkArea, MonitorWorkArea
WorkAreaHeight := WorkAreaBottom - WorkAreaTop
IniRead, ClickThrough, %IniPath%, General, ClickThrough, 1
IniRead, AlwOnTop, %IniPath%, General, AlwaysOnTop, 1
IniRead, Transparenz, %IniPath%, General, Transparenz, 7
IniRead, PrefDay, %IniPath%, General, PrefWeekDay, 0
IniRead, Reboot, %IniPath%, General, Reboot, 0
IniRead, BeepStop, %IniPath%, General, BeepStop, 5
IniRead, NoConfirm, %IniPath%, General, NoConfirm, 0
IniRead, AlwTrayIcon, %IniPath%, General, AlwTrayIcon, 1
IniRead, xPos, %IniPath%, General, WinPosX, % A_ScreenWidth - 126
IniRead, yPos, %IniPath%, General, WinPosY, % WorkAreaHeight - 126
Loop, 9
  {
  IniRead, AlarmCheck%A_Index%, %IniPath%, Alarm%A_Index%, AlarmCheck%A_Index%, %A_Space%
  IniRead, AlarmName%A_Index%, %IniPath%, Alarm%A_Index%, AlarmName%A_Index%, %A_Space%
  IniRead, AlarmTime%A_Index%, %IniPath%, Alarm%A_Index%, AlarmTime%A_Index%, %A_Space%
  IniRead, AlarmDate%A_Index%, %IniPath%, Alarm%A_Index%, AlarmDate%A_Index%, %A_Space%
  IniRead, AlarmRerun%A_Index%, %IniPath%, Alarm%A_Index%, AlarmRerun%A_Index%, %A_Space%
  IniRead, AlarmAction%A_Index%, %IniPath%, Alarm%A_Index%, AlarmAction%A_Index%, %A_Space%
  IniRead, AlarmExec%A_Index%, %IniPath%, Alarm%A_Index%, AlarmExec%A_Index%, %A_Space%
  AlarmTime%A_Index% := A_YYYY . A_MM . A_DD . AlarmTime%A_Index% ; vollwertigen TimeStamp erstellen
  }
If ClickThrough
  {
  Menu, Tray, Check, %lng_ClickThrough%
  Menu, Tray, Default, &%lng_Alarms%... ; ALARMS
  }
If (AlwTrayIcon || ClickThrough)
  Menu, Tray, Icon

If !pToken := Gdip_Startup() ; Start gdi+
  {
  MsgBox, 48, %ScriptName%, %lng_Gdiplus%
  ExitApp
  }
IfNotExist, %ScriptNoExt%.png
  {
  SplashImage,, CWeeeeee b1 FS10 w300, %lng_Download%,,, MS Sans Serif
  Sleep, 10 ; otherwise no text visible
  URLDownloadToFile, https://www.ophthalmostar.de/wp-content/uploads/AHKClock.png, %ScriptNoExt%.png
  SplashImage, Off
  }
; ------------------------------------------------------------------------------
Gui, 1: +LastFound -Caption +E0x80000 +ToolWindow ; +0x02000000 ; +AlwaysOnTop ; +OwnDialogs
Gui1ID := WinExist() ; Get a handle to this window
Gui, 1: Show, NA ; Show the window
pBitmap := Gdip_CreateBitmapFromFile(ScriptNoExt . ".png") ; Get a bitmap from the image
If !pBitmap ; in case the file was corrupt
  {
  MsgBox, 48, %ScriptName%, %lng_NoPicture%
  FileDelete, %ScriptNoExt%.png ; delete 0-KB-file from broken download
  ExitApp
  }
iWidth := Gdip_GetImageWidth(pBitmap), iHeight := Gdip_GetImageHeight(pBitmap)
xPos := (xPos > A_ScreenWidth - iWidth) ? A_ScreenWidth - iWidth : (xPos < 0) ? 0 : xPos
yPos := (yPos > WorkAreaHeight - iHeight) ? WorkAreaHeight - iHeight : (yPos < 0) ? 0 : yPos
hbm := CreateDIBSection(iWidth, iHeight) ; Create a gdi bitmap
hdc := CreateCompatibleDC() ; device context compatible with the screen
obm := SelectObject(hdc, hbm) ; Select the bitmap into the device context
G := Gdip_GraphicsFromHDC(hdc) ; Get a pointer for use with drawing functions
Gdip_SetCompositingMode(G, 1) ; Set to overdraw
Gdip_DrawImage(G, pBitmap, 0, 0, iWidth, iHeight, 0, 0, iWidth, iHeight) ; draw the bitmap
TransNorm := 1 <= Transparenz <= 9 ? Round(Transparenz * 25.5) : 255
UpdateLayeredWindow(Gui1ID, hdc, xPos, yPos, iWidth, iHeight, TransNorm) ; Update the window
hCurs:=DllCall("LoadCursor", "UInt", NULL, "Int", 32649, "UInt") ;IDC_HAND
OnMessage(0x201, "WM_LBUTTONDOWN") ; OnMessage will let us drag the clock
OnMessage(0x203, "WM_LBUTTONDBLCLK")
OnMessage(0x200, "WM_MOUSEMOVE")
OnMessage(0x404, "AHK_NOTIFYICON")
Diameter := 119 ; 118
CenterX := CenterY := floor(Diameter/2) ; Center x
If (AlwOnTop) ; führte weiter oben zu Fehlern
    Gui, 1: +AlwaysOnTop

GoSub, ClockCheck ; Zeiger einzeichnen
SetTimer, ClockCheck, 1000
GoSub, InitializeTimers

Menu, ct_1Menu, Add, &%lng_About%..., ABOUT
Menu, ct_1Menu, Add
Menu, ct_1Menu, Add, &%lng_Alarms%..., ALARMS
Menu, ct_1Menu, Add, &%lng_Options%..., OPTIONEN
Menu, ct_1Menu, Add, %lng_HideClock%, HideShowGui
Menu, ct_1Menu, Add, %lng_ClickThrough%, NotClickable
Menu, ct_1Menu, Add
If (!A_IsCompiled)
  {
  Menu, AutoHotkey, Standard
  Menu, ct_1Menu, Add, AutoHotkey, :AutoHotkey
  Menu, ct_1Menu, Add
  }
Menu, ct_1Menu, Add, &%lng_Exit%, PROGEXIT

Menu, ct_3Menu, Add, &%lng_Edit%`tF2, ALARMCHANGE
Menu, ct_3Menu, Add, &%lng_Duplicate%`tF3, ALARMDUPL
Menu, ct_3Menu, Add, &%lng_Delete%`t%lng_Del%, ALARMDELETE
Menu, ct_3Menu, Add
Menu, ct_3Menu, Add, &%lng_DoItNow%`tStrg+Enter, ALARMEXEC
Menu, ct_3Menu, Add
Menu, ct_3Menu, Add, &%lng_Properties%`tAlt+Enter, PROPERTIES
Return
; ============= Ende der bisher erledigten Lokalisierung =======================

GUIESCAPE: ; released by Hotkey or Escape after Mouseclick on the window
HideShowGui:
  If (DllCall("IsWindowVisible", "Ptr", Gui1ID))
    {
    SetTimer, REMOVETT, -10
  WinGet, ExStyle, ExStyle, ahk_id %Gui1ID%  ; 0x8 is WS_EX_TOPMOST.
  If !(ExStyle & 0x8)
    WinSet, AlwaysOnTop, Off, ahk_id %Gui1ID% ; Gui, 1: +AlwaysOnTop
    Gui, Hide
    SetTimer, ClockCheck, Off
    Menu, Tray, UnCheck, %MenuItemShowHide%
    If (AlwTrayIcon || ClickThrough)
      Menu, Tray, Icon
    }
  Else
    {
    Gui, Show, NoActivate
    SetTimer, ClockCheck, 1000
    If (AlwOnTop)
    WinSet, AlwaysOnTop, On, ahk_id %Gui1ID% ; Gui, 1: +AlwaysOnTop
    Menu, Tray, Check, %MenuItemShowHide%
    If !(AlwTrayIcon || ClickThrough)
      Menu, Tray, NoIcon
    GoSub, ClockCheck
    }
Return

; _____________________________________________________________________________
; ——————————————————————————————————————————————————————————————————————————————
; ||  L O O P  //
; ——————————————
; ¯¯¯¯¯¯¯¯¯¯¯¯¯
ClockCheck:
  Gdip_SetCompositingMode(G, 1) ; set to overdraw
  Gdip_DrawImage(G, pBitmap, 0, 0, Diameter, Diameter, 0, 0, Diameter, Diameter)
  Gdip_SetCompositingMode(G, 0) ; switch off overdraw
  Gdip_SetSmoothingMode(G, 4)  ; Variable: AntiAlias ; turn on antialiasing
  t := A_Min * 360 // 60 + 90
  DrawHands(t, 0.2, 0.92, 2.5, ColorMH) ; time, outer position, inner position, thickness
  t := A_Hour * 360 // 12 + (A_Min * 360 // 60) // 12 + 90
  DrawHands(t, 0.4, 0.92, 3, ColorMH) ; time, outer position, inner position, thickness
  t := A_Sec * 360 // 60 + 90 ; Draw seconds hand
  DrawHands(t, 0.25, 0.94, 1.2, Color_S) ; time, outer position, inner position, thickness
  t := A_Sec * 360 // 60 - 90            ; TALE - Zeile löschen wenn nicht gewünscht
  DrawHands(t, 0.75, 0.94, 1.2, Color_S) ; TALE - Zeile löschen wenn nicht gewünscht
  Gdip_SetSmoothingMode(G, 0) ; turn off antialiasing
  Foo := TransTemp ? TransTemp : TransNorm
  UpdateLayeredWindow(Gui1ID, hdc, "", "", "", "", Foo)
Return

DateAndTime:
;  Run, %A_WinDir%\explorer.exe shell:::{E2E7934B-DCE5-43C4-9576-7FE4F75E7480},, UseErrorLevel ; Opens Date and Time
 Run, %A_WinDir%\System32\rundll32.exe shell32.dll`,Control_RunDLL timedate.cpl`,`,0,, UseErrorLevel
  If ErrorLevel = ERROR
      MsgBox, 48, %ScriptName%, «Datum und Uhrzeit» konnte nicht geöffnet werden.
Return

NotClickable:
  ClickThrough := !ClickThrough
  If ClickThrough
    {
    Menu, Tray, Check, %lng_ClickThrough%
    If !(AlwTrayIcon)
      Menu, Tray, Icon
    Menu, Tray, Default, &%lng_Alarms%... ; ALARMS
    TrayTip, %ScriptName%, Linksklick öffnet oder schließt die Alarm-Liste., 20, 17
    }
  Else
    {
    Menu, Tray, UnCheck, %lng_ClickThrough%
    Menu, Tray, Default, %MenuItemShowHide% ; &Uhr anzeigen
    TrayTip, %ScriptName%, Linksklick zeigt die Uhr an oder blendet sie aus., 20, 17
    }
  IniWrite, %ClickThrough%, %IniPath%, General, ClickThrough
Return

InitializeTimers:
  Loop, 9
    {
    If (AlarmCheck%A_Index%) ; && AlarmRerun%A_Index% = 4) ; Timer für abgeleitete Zeiten in der Zukunft starten
      ResetTimer(A_Index) ; löscht abgelaufene einmalige Aufgaben (AlarmRerun = 1)
    }
Return

AlarmTimer1:
  SetTimer, AlarmTimer1, Off
  ACTION_%AlarmAction1%(AlarmName1, AlarmRerun1, AlarmExec1, RegExReplace(Fulltext(AlarmRerun1), "^([A-Z])", "$L1"))
  ActionTime1 := A_Now
  ResetTimer(1)
Return

AlarmTimer2:
  SetTimer, AlarmTimer2, Off
  ACTION_%AlarmAction2%(AlarmName2, AlarmRerun2, AlarmExec2, RegExReplace(Fulltext(AlarmRerun2), "^([A-Z])", "$L1"))
  ActionTime2 := A_Now
  ResetTimer(2)
Return

AlarmTimer3:
  SetTimer, AlarmTimer3, Off
  ACTION_%AlarmAction3%(AlarmName3, AlarmRerun3, AlarmExec3, RegExReplace(Fulltext(AlarmRerun3), "^([A-Z])", "$L1"))
  ActionTime3 := A_Now
  ResetTimer(3)
Return

AlarmTimer4:
  SetTimer, AlarmTimer4, Off
  ACTION_%AlarmAction4%(AlarmName4, AlarmRerun4, AlarmExec4, RegExReplace(Fulltext(AlarmRerun4), "^([A-Z])", "$L1"))
  ActionTime4 := A_Now
  ResetTimer(4)
Return

AlarmTimer5:
  SetTimer, AlarmTimer5, Off
  ACTION_%AlarmAction5%(AlarmName5, AlarmRerun5, AlarmExec5, RegExReplace(Fulltext(AlarmRerun5), "^([A-Z])", "$L1"))
  ActionTime5 := A_Now
  ResetTimer(5)
Return

AlarmTimer6:
  SetTimer, AlarmTimer6, Off
  ACTION_%AlarmAction6%(AlarmName6, AlarmRerun6, AlarmExec6, RegExReplace(Fulltext(AlarmRerun6), "^([A-Z])", "$L1"))
  ActionTime6 := A_Now
  ResetTimer(6)
Return

AlarmTimer7:
  SetTimer, AlarmTimer7, Off
  ACTION_%AlarmAction7%(AlarmName7, AlarmRerun7, AlarmExec7, RegExReplace(Fulltext(AlarmRerun7), "^([A-Z])", "$L1"))
  ActionTime7 := A_Now
  ResetTimer(7)
Return

AlarmTimer8:
  SetTimer, AlarmTimer8, Off
  ACTION_%AlarmAction8%(AlarmName8, AlarmRerun8, AlarmExec8, RegExReplace(Fulltext(AlarmRerun8), "^([A-Z])", "$L1"))
  ActionTime8 := A_Now
  ResetTimer(8)
Return

AlarmTimer9:
  SetTimer, AlarmTimer9, Off
  ACTION_%AlarmAction9%(AlarmName9, AlarmRerun9, AlarmExec9, RegExReplace(Fulltext(AlarmRerun9), "^([A-Z])", "$L1"))
  ActionTime9 := A_Now
  ResetTimer(9)
Return

EXITPROG:
  If (DllCall("IsWindowVisible", "Ptr", Gui1ID))
  {
    WinGetPos, xPos, yPos,,, ahk_id %Gui1ID%
    IniWrite, %xPos%, %IniPath%, General, WinPosX
    IniWrite, %yPos%, %IniPath%, General, WinPosY
  }
  SelectObject(hdc, obm) ; Select the object back into the hdc
  DeleteObject(hbm) ; Now the bitmap may be deleted
  DeleteDC(hdc) ; Also the device context related to the bitmap
  Gdip_DeleteGraphics(G) ; The graphics may now be deleted
  Gdip_DisposeImage(pBitmap) ; Now the bitmap we made from the image
  Gdip_Shutdown(pToken)
PROGEXIT:
  ExitApp
Return

SPLASHOFF:
  SplashImage, Off
Return

SHOWTIME:
  MouseGetPos,,, foo
  If foo <> %Gui1ID%
    Return
  FormatTime, bar,, 'Datum:`t' dd.MM.yy
  FormatTime, baz,, 'Zeit:`t' HH:mm:ss
  ToolTip, % baz "`n" bar ;, % xPos + 15, % yPos + 126
Process, Exist
tThWnd1 := WinExist("ahk_class tooltips_class32 ahk_pid " ErrorLevel)
  SetTimer, REMOVETT, -3000
Return

REMOVETT:
  ToolTip
  tThWnd1 =
Return

GuiContextMenu:
  Menu, ct_1Menu, Show
Return

; _____________________________________________________________________________
; ——————————————————————————————————————————————————————————————————————————————
; ||  E I E R U H R  //
; ————————————————————————————
; ¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯
EggTimer:
  menuItem := A_ThisMenuItem
  numMin := RegExReplace(menuItem, "\D")  ; any non-digit
  For index, value in arrayEggMenu
    Menu, SubMenu, UnCheck, %value%
  If (menuItem <> prevItem)  ; if (numMin > 1 && ...
  {
    prevItem := menuItem
    Menu, SubMenu, Check, %menuItem%
    SetTimer, EggTimerMsg, % -(numMin * 60000)
  }
  Else  ; d.h. menuItem = prevItem => nur UnChecken
  {
    SetTimer, EggTimerMsg, Off
    prevItem =  ; Checking für menuItem ermöglichen
  }
Return

EggTimerMsg: ; Meldung
  Menu, SubMenu, UnCheck, %menuItem%
  prevItem =  ; Checking für menuItem ermöglichen
  MsgBox, 262192, %lng_EggTimer%, %menuItem% sind abgelaufen.
Return

; _____________________________________________________________________________
; ——————————————————————————————————————————————————————————————————————————————
; ||  O P T I O N E N  //
; ————————————————————————————
; ¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯
OPTIONEN:
  Suspend, On
;  _winkey := (InStr(SubStr(_hotkey, 1, -1), "#")) ? 1 : 0
;  _tmpkey := _winkey ? RegExReplace(_hotkey, "#", "" ,"" , 1) : _hotkey ; 1x # entfernen
  Gui, 2: Destroy
  Gui, 2: -MinimizeBox -MaximizeBox ; +LastFound
  Gui, 2: Margin, 5, 5
  Gui, 2: Font, s9 norm, MS Sans Serif
  Gui, 2: Add, Tab3, w330 h205, Allgemein|Analoguhr|Alarm-Liste
  Gui, 2: Tab, 1
  Gui, 2: Add, Checkbox, xm+10 ym+40 vAutoStart gUACProblem Checked, &Automatischer Start beim Anmelden des Windows-Benutzers
  IfNotExist, %A_Startup%\%ScriptNoExt%.lnk
      GuiControl, 2:, AutoStart, 0
  Gui, 2: Add, Checkbox, xm+10 yp+35 vAlwTrayIcon Checked%AlwTrayIcon%, &Symbol im Infobereich der Taskleiste immer anzeigen
  If ClickThrough
    {
    GuiControl, 2:, AlwTrayIcon, 1
    GuiControl, 2: Disable, AlwTrayIcon
    }
  Gui, 2: Add, GroupBox, xm+10 yp+35 w310 h85, &Tipps und Tricks
; Gui, 2: Add, Text, xm+17 yp+20, Das Programm benötigt Administratorrechte. Um nicht jedes-`nmal den UAC-Dialog bestätigen zu müssen`, könnnen Sie das`nProgramm über die Aufgabenplanung automatisch starten.`nAktivieren Sie die Option «Mit höchsten Privilegien ausführen».
  Gui, 2: Add, Text, xm+20 yp+20, Wenn Sie mit der linken Maustaste auf das Symbol im Info-`nbereich der Taskleiste klicken`, wird die Uhr angezeigt oder`nversteckt.  Wenn «Hindurchklickbar» aktiv ist, wird das Sym-`nbol im Infobereich der Taskleiste in jedem Fall angezeigt.
; Nach Betätigung der rechten Maustaste öffnet sich ein Menü.
;  Gui, 2: Add, GroupBox, xm+10 yp+35 w310 h85, &Tastaturkürzel
;  Gui, 2: Add, Text, xm+20 yp+20, Mit diesem Hotkey wird die Uhr angezeigt oder ausgeblendet.`nDrücken Sie eine Tastenkombination, um ihn zu ändern.
;  Gui, 2: Add, Hotkey, xm+20 yp+35 w195 v_tmpkey, %_tmpkey%
;  Gui, 2: Add, Checkbox, x+5 yp+3 v_winkey Checked%_winkey%, zus. &Win-Taste
  Gui, 2: Tab, 2
  Gui, 2: Add, Checkbox, xm+10 ym+40 vAlwOnTop Checked%AlwOnTop%, Uhr über anderen Fenstern anzeigen (&AlwaysOnTop)
  Gui, 2: Add, Text, xm+10 yp+35, Trans&parenz (je höher`, desto durchsichtiger):
  Gui, 2: Add, DropDownList, x+5 yp-4 vTransparenz AltSubmit Choose%Transparenz% W60, 90`%|80`%|70`%|60`%|50`%|40`%|30`%|20`%|10`%|Aus
  Gui, 2: Add, GroupBox, xm+10 yp+39 w310 h85, &Tipps und Tricks
  Gui, 2: Add, Text, xm+20 yp+20, Wenn «Hindurchklickbar»
  Gui, 2: Font, Underline
  Gui, 2: Add, Text, x+3 yp, nicht
  Gui, 2: Font, Norm
  Gui, 2: Add, Text, x+3 yp, aktiv ist, können Sie
  Gui, 2: Add, Text, xm+20 yp+20, - die Uhr mit gedrückter linker Maustaste verschieben und
  Gui, 2: Add, Text, xm+20 yp+20, - die Alarmliste mit einem Doppelklick auf die Uhr öffnen.
  Gui, 2: Tab, 3
  Gui, 2: Add, Text, xm+10 ym+40, Aktion «&Ton» nach
  Gui, 2: Add,  Edit, xm+108 yp-2 w45 Limit2 vDuration ; Number nicht erford. weil UpDown folgt
  Gui, 2: Add, UpDown, vBeepStop Range1-60, %BeepStop%
  Gui, 2: Add, Text, x+6 yp+2, Minuten automatisch beenden
  Gui, 2: Add, Checkbox, xm+10 yp+35 vReboot Checked%Reboot%, Bei Durchführung der Aktion «&Herunterfahren» neustarten
  Gui, 2: Add, Checkbox, xm+10 yp+35 vNoConfirm Checked%NoConfirm%, Aktion «&Programm» ohne Bestätigung unmittelbar ausführen

  Gui, 2: Add, Checkbox, xm+10 yp+35 vPrefDay Checked%PrefDay%, &Gleichem Wochentag statt gleichem Datum Vorrang geben
  Gui, 2: Add, Text, xm+29 yp+20, Diese Option ermöglicht die Wiederholung an bestimmten`nWochentagen`, z.B. an jedem zweitem Montag im Monat.

  Gui, 2: Tab
  Gui, 2: Add, Button, gSETTINGSOK Default xm+145 ym+210 w90, &OK ; xm+115
  Gui, 2: Add, Button, g2GUICLOSE x+5 W90, &Abbrechen
  Gui, 2: Show,, %ScriptName% %lng_Options%

Return
;------------------------------------------------------------------------------

SETTINGSOK:
  Gui, 2: Submit, NoHide
  GoSub, 2GUICLOSE
  BeepStop := Duration
  IniWrite, %AlwOnTop%, %IniPath%, General, AlwaysOnTop
  IniWrite, %Transparenz%, %IniPath%, General, Transparenz
  IniWrite, %PrefDay%, %IniPath%, General, PrefWeekDay
  IniWrite, %Reboot%, %IniPath%, General, Reboot
  IniWrite, %BeepStop%, %IniPath%, General, BeepStop
  IniWrite, %NoConfirm%, %IniPath%, General, NoConfirm
  IniWrite, %AlwTrayIcon%, %IniPath%, General, AlwTrayIcon
  If !(AlwTrayIcon || ClickThrough)
  {
    If (DllCall("IsWindowVisible", "Ptr", Gui1ID))
      Menu, Tray, NoIcon
    Else ; wenn während der Anzeige der Optionen Uhr verborgen wurde
      Menu, Tray, Icon
  }
  Else
    Menu, Tray, Icon
  If (AutoStart)
    {
    IfNotExist, %A_Startup%\%ScriptNoExt%.lnk
      {
      FileCreateShortcut, %A_ScriptFullPath%, %A_Startup%\%ScriptNoExt%.lnk, %A_ScriptDir%,, %ScriptName%
      If (ErrorLevel)
        MsgBox, 16, %ScriptNoExt%, Die Autostart-Verknüpfung konnte nicht erstellt werden.
      }
    }
  Else
    {
    FileDelete, %A_Startup%\%ScriptNoExt%.lnk
    If ErrorLevel = 0
      MsgBox, 64, %ScriptNoExt%, %A_Startup%\%ScriptNoExt%.lnk wurde gelöscht.
    }
  If (AlwOnTop)
    WinSet, AlwaysOnTop, On, ahk_id %Gui1ID% ; Gui, 1: +AlwaysOnTop
  Else ; nur -AlwaysOnTop
    WinSet, AlwaysOnTop, Off, ahk_id %Gui1ID% ; Gui, 1: -AlwaysOnTop
 TransNorm := 1 <= Transparenz <= 9 ? Round(Transparenz * 25.5) : 255
  UpdateLayeredWindow(Gui1ID, hdc, "", "", "", "", TransNorm)
Return

2GUICLOSE:
2GUIESCAPE:
  Suspend, Off
  Gui, 2: Destroy
Return

UACProblem:
  Gui, 2: Submit, NoHide
  Gui, 2: +OwnDialogs ; wg. MsgBox
  If (AutoStart && !RegExMatch(A_OSVersion, "^[WIN_NT4|WIN_95|WIN_98|WIN_ME|WIN_2000|WIN_XP|WIN_2003]+$")) ; WIN_VISTA und höher
    {
    MsgBox, 36, Bitte beachten Sie:,  Falls als Alarm-Aufgabe Programme gestartet werden sollen, be-`nnötigt das Programm Administratorrechte. Um nicht jedesmal den`nUAC-Dialog bestätigen zu müssen`, könnnen Sie %ScriptNoExt% über die`nAufgabenplanung starten`n(Bei Anmeldung starten, Mit höchsten Berechtigungen ausführen).`n`nSoll die Aufgabenplanung jetzt geöffnet werden?
    IfMsgBox, Yes
      {
      Run, taskschd.msc,, UseErrorLevel
      If ErrorLevel = Error
        MsgBox,16, %ScriptName% %ScriptVersion%, Die Aufgabenplanung kann nicht geöffnet werden!
      Else
        GuiControl, 2:, AutoStart, 0
      }
    }
Return

; _____________________________________________________________________________
; ——————————————————————————————————————————————————————————————————————————————
; ||  A L A R M S  //
; ————————————————————————————
; ¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯
ALARMS:
  Gui 3: +LastFoundExist
  IfWinExist
    {
      Gosub, 3GUICLOSE
      Return
    }
  Gui, 3: Destroy
  Loop, 9
    SetTimer, AlarmTimer%A_Index%, Off
  foo =
  Loop, 9
    {
    foo ++
    If (!AlarmName%A_Index%)
      {
      foo ++
      AlarmCheck%A_Index% := AlarmCheck%foo%
      AlarmName%A_Index%  := AlarmName%foo%
      AlarmTime%A_Index%  := AlarmTime%foo%
      AlarmDate%A_Index%  := AlarmDate%foo%
      AlarmRerun%A_Index% := AlarmRerun%foo%
      AlarmAction%A_Index% := AlarmAction%foo%
      AlarmExec%A_Index% := AlarmExec%foo%
      AlarmCheck%foo% =
      AlarmName%foo% =
      AlarmTime%foo% =
      AlarmDate%foo% =
      AlarmRerun%foo% =
      AlarmAction%foo% =
      AlarmExec%foo% =
      }
    }
  Loop, 9  ; alle Alarm-Sektionen löschen
    IniDelete, %IniPath%, % "Alarm" . A_Index
  Loop, 9
    {
    If (AlarmName%A_Index%) ; keine Leereinträge
      {
      IniWrite, % AlarmCheck%A_Index% , %IniPath%, % "Alarm" . A_Index, AlarmCheck%A_Index%
      IniWrite, % AlarmName%A_Index%  , %IniPath%, % "Alarm" . A_Index, AlarmName%A_Index%
      IniWrite, % SubStr((AlarmTime%A_Index%), -5), %IniPath%, % "Alarm" . A_Index, AlarmTime%A_Index%
      IniWrite, % SubStr(AlarmDate%A_Index%, 1, 8), %IniPath%, % "Alarm" . A_Index,  AlarmDate%A_Index%
      IniWrite, % AlarmRerun%A_Index% , %IniPath%, % "Alarm" . A_Index, AlarmRerun%A_Index%
      IniWrite, % AlarmAction%A_Index%, %IniPath%, % "Alarm" . A_Index, AlarmAction%A_Index%
      IniWrite, % AlarmExec%A_Index%  , %IniPath%, % "Alarm" . A_Index, AlarmExec%A_Index%
      }
    }
  Gui, 3: +LastFound -MinimizeBox -MaximizeBox
  Gui3ID := WinExist()
  GroupAdd, GroupList, ahk_id %Gui3ID%
  Gui, 3: Default ; sonst funktioniert LV_Add nicht
  Gui, 3: Margin, 10, 10
  Gui, 3: Font, s9 norm, MS Sans Serif
; NoSortHdr ist erforderlich, weil Sortierung NICHT funktioniert! (s. Properties und andere)
  Gui, 3: Add, ListView, r9 w330 vAlarm_LV gALARM_LV NoSortHdr Checked ReadOnly -Multi Grid HwndLV_ID, Name|Uhrzeit|Ausführung
  LV_ModifyCol(1, "185")
  LV_ModifyCol(2, "61")
  LV_ModifyCol(3, "80")
  Loop, 9
    {
    If (AlarmName%A_Index%)
      {
      FormatTime, foo, % AlarmTime%A_Index%, HH:mm:ss ; Time - nur Stunden und Minuten, keine Sekunden
      LV_Add("Check", AlarmName%A_Index%, foo, Fulltext(AlarmRerun%A_Index%))
      If (!AlarmCheck%A_Index%)
        LV_Modify(A_Index, "-Check")
      }
    }
  Gui, 3: Add, Button, gALARMNEW xm+340 ym w80 h22, &Hinzufügen...
  Gui, 3: Add, Button, gALARMCHANGE xp yp+30 w80 h22, &Bearbeiten...
  Gui, 3: Add, Button, gALARMDELETE xp yp+30 w80 h22, &Löschen...
; Gui, 3: Add, Button, gALARMDUPL xp yp+30 w80 h22, &Duplizieren...
  Gui, 3: Add, Button, gALARMEXEC xp yp+30 w80 h22, &Testen
  Gui, 3: Add, Button, gPROPERTIES xp yp+30 w80 h22, &%lng_Properties%
  Gui, 3: Add, Button, gALLSAVE xp yp+30 w80 h22 Default, &Schließen
  Gui, 3: Show, AutoSize Center, Alarm-Liste
Return

3GuiContextMenu:
  If A_GuiControl <> Alarm_LV
    Return
  AlarmRow := LV_GetNext()
  If AlarmRow ; LV_GetNext() ; If A_EventInfo zeigt auch Menu bei leeren Zeilen
    Menu, ct_3Menu, Show, %A_GuiX%, %A_GuiY%
Return

NOENTRY:
    WinGetPos, gPosX, gPosY, gWidth, gHeight, ahk_id %Gui3ID%
    gPosX := gPosX - 125 + (gWidth / 2)
    gPosY := gPosY - 15 + (gHeight / 2)
    SplashImage,, CWeeeeee b1 FS10 w200 x%gPosX% y%gPosY%, Es ist kein Eintrag markiert! ,,, MS Sans Serif
    SetTimer, SPLASHOFF, -800
Return

PROPERTIES:
  Gui, 3: +OwnDialogs
  Gui, 3: Default ; sonst funktioniert 3GuiContextMenu nicht
  AlarmRow := LV_GetNext()
  If (!AlarmRow)
    {
   GoSub, NOENTRY
       Return
    }
  MsgBox, 32, Alarm-Eigenschaften, % RegExReplace(GetAlarmInfo(AlarmRow), "`n", "`n`n")
Return

GetAlarmInfo(AIndex)
  {
  bar := AlarmAction%AIndex% = 1 ? "Ton" : AlarmAction%AIndex% = 2 ? "Meldung" : AlarmAction%AIndex% = 3 ? Reboot = 1 ? "Neustarten (s. Einstellungen)" : "Herunterfahren" : AlarmExec%AIndex%
  baz := AlarmRerun%AIndex% = 1 ? "`nDie Aufgabe wird nach der Ausführung gelöscht." : ""
  Rerun := Fulltext(AlarmRerun%AIndex%)
  FormatTime, AlarmWDay, % AlarmDate%AIndex%, dddd
  Rerun := PrefDay And AlarmRerun%AIndex% > 3 ? Rerun " (am " AlarmWDay "`, s. Einstellungen)" : Rerun
  foo := AlarmRerun%AIndex%
  AlarmDT := SubStr(AlarmDate%AIndex%, 1, 8) . SubStr((AlarmTime%AIndex%), -5)
  FormatTime, StartDT, %AlarmDT%, dd.MM.yy, HH:mm:ss
  NextDate := (LV_GetNext(AIndex - 1, "Checked") = AIndex) ? FormatNextExec(NextExec%foo%(AlarmDT)) : "ist nicht aktiviert"
  return % "Name: `t" AlarmName%AIndex% "`nModus: `t" Rerun "`nBasis: `t" StartDT "`nTermin: `t" NextDate "`nAktion: `t" bar  baz
  }

ALARMEXEC:
  Gui, 3: +OwnDialogs
  Gui, 3: Default ; sonst funktioniert 3GuiContextMenu nicht
  AlarmRow := LV_GetNext()
  If (!AlarmRow)
    {
    GoSub, NOENTRY
    Return
    }
  AIndex := AlarmAction%AlarmRow%
  AName := AlarmName%AlarmRow%
  ARerun := AlarmRerun%AlarmRow%
  AExec := AlarmExec%AlarmRow%
  TxtRerun := RegExReplace(Fulltext(ARerun), "^([A-Z])", "$L1") ; erster Buchstabe klein
  ACTION_%AIndex%(AName, ARerun, AExec, TxtRerun)
Return

ALARM_LV:
  If A_GuiEvent = DoubleClick
    {
    If LV_GetNext() ; If A_EventInfo zeigt auch Menu bei leeren Zeilen
      GoSub, ALARMCHANGE
    Else
      GoSub, ALARMNEW
    }
Return

ALARMDELETE:
  Gui, 3: Default ; sonst funktioniert 3GuiContextMenu nicht
  AlarmRow := LV_GetNext()
  If (!AlarmRow)
    {
   GoSub, NOENTRY
       Return
    }
  MsgBox, 36, %ScriptName%, % "Soll der folgende Alarm gelöscht werden?`n`n" . GetAlarmInfo(AlarmRow)
  IfMsgBox, No
    Return
  LV_Delete(AlarmRow)
  If (AlarmRow = 9)
    {
    AlarmCheck%AlarmRow% =
    AlarmName%AlarmRow% =
    AlarmTime%AlarmRow% =
    AlarmDate%AlarmRow% =
    AlarmRerun%AlarmRow% =
    AlarmAction%AlarmRow% =
    AlarmExec%AlarmRow% =
    }
  Else
    {
    Loop, 9
      {
      If (A_Index > AlarmRow) ; also keinesfalls wenn AlarmRow = 9
        {
        foo := A_Index - 1
        AlarmCheck%foo%  := AlarmCheck%A_Index%
        AlarmName%foo%   := AlarmName%A_Index%
        AlarmTime%foo%   := AlarmTime%A_Index%
        AlarmDate%foo%   := AlarmDate%A_Index%
        AlarmRerun%foo% := AlarmRerun%A_Index%
        AlarmAction%foo% := AlarmAction%A_Index%
        AlarmExec%foo% := AlarmExec%A_Index%
        AlarmCheck%A_Index% =
        AlarmName%A_Index% =
        AlarmTime%A_Index% =
        AlarmDate%A_Index% =
        AlarmRerun%A_Index% =
        AlarmAction%A_Index% =
        AlarmExec%A_Index% =
        }
      }
    }
Return

ALARMCHANGE:
  Gui, 3: Default ; sonst funktioniert 3GuiContextMenu nicht
  AlarmRow := LV_GetNext()
  If (!AlarmRow)
    {
   GoSub, NOENTRY
       Return
    }
  AlarmName := AlarmName%AlarmRow%
  AlarmTime := AlarmTime%AlarmRow%
  AlarmDate := AlarmDate%AlarmRow%
  AlarmRerun := AlarmRerun%AlarmRow%
  AlarmAction := AlarmAction%AlarmRow%
  AlarmExec := AlarmExec%AlarmRow%
  EditorTitle = %lng_Edit%
  GoSub, ALARMEDITOR
Return

ALARMNEW:
  Gui, 3: +OwnDialogs
  Gui, 3: Default ; sonst funktioniert 3GuiContextMenu nicht
If (LV_GetCount() = 9)
    {
      MsgBox, 48, Alarm-Liste, Es können maximal 9 Einträge verwaltet werden.
      Return
    }
  AlarmRow =
  AlarmName = Geben Sie hier eine Benennung an.
  AlarmTime =
  AlarmDate =
  AlarmRerun =
  AlarmAction =
  AlarmExec =
  EditorTitle = %lng_New%
  GoSub, ALARMEDITOR
Return

ALARMDUPL:
  Gui, 3: +OwnDialogs
  Gui, 3: Default ; sonst funktioniert 3GuiContextMenu nicht
  AlarmRow := LV_GetNext()
  If (!AlarmRow)
    {
   GoSub, NOENTRY
       Return
    }
  If (LV_GetCount() = 9)
    {
      MsgBox, 48, Alarm-Liste, Es können maximal 9 Einträge verwaltet werden.
      Return
    }
  AlarmRow := LV_GetNext()
  AlarmName := AlarmName%AlarmRow%
  AlarmTime := AlarmTime%AlarmRow%
  AlarmDate := AlarmDate%AlarmRow%
  AlarmRerun := AlarmRerun%AlarmRow%
  AlarmAction := AlarmAction%AlarmRow%
  AlarmExec := AlarmExec%AlarmRow%
  AlarmRow =
  EditorTitle = %lng_Duplicate%
  GoSub, ALARMEDITOR
Return

ALARMEDITOR:
  Gui, 4: Destroy
  Gui, 4: +Owner3
  Gui, 3: +Disabled
  Gui, 4: +LastFound -MinimizeBox -MaximizeBox
  Gui4ID := WinExist()
  GroupAdd, GroupEdit, ahk_id %Gui4ID%
  Gui, 4: Margin, 10, 10
  Gui, 4: Font, s9 norm, MS Sans Serif
  Gui, 4: Add, GroupBox, xm ym Section w310 h195, &Name
  Gui, 4: Add, Edit, xm+10 ys+22 w290 r1 vAlarmName Limit80, %AlarmName%
  Gui, 4: Add, GroupBox, xm ym+55 Section w310 h140, &Zeit (Um die akutelle Zeit einzustellen, drücken Sie F5)
  Gui, 4: Add, DateTime, xm+10 ys+22 w80 vAlarmTime, Time ; HH:mm:ss bewirkt DropDown mit Kalender
  GuiControl, 4:, AlarmTime, %AlarmTime%
  Gui, 4: Add, DateTime, xm+100 yp w90 vAlarmDate
  GuiControl, 4:, AlarmDate, %AlarmDate%
  AlarmRerun := !AlarmRerun ? 1 : AlarmRerun
  Gui, 4: Add, DropDownList, xm+200 yp w100 vAlarmRerun AltSubmit Choose%AlarmRerun%, % Fulltext(1) "|" Fulltext(2) "|" Fulltext(3) "|" Fulltext(4) "|" Fulltext(5) "|" Fulltext(6) "|" Fulltext(7)
  Gui, 4: Add, GroupBox, xm ym+110 Section w310 h85, &Aktion
  AlarmBeep := AlarmAction = 1 ? 1 : 0
  Gui, 4: Add, Radio, xm+10 ys+23 vAlarmBeep gALARMPROG Checked%AlarmBeep%, Ton
  AlarmMessage := AlarmAction = 2 ? 1 : !AlarmAction ? 1 : 0
  Gui, 4: Add, Radio, xm+100 yp vAlarmMessage gALARMPROG Checked%AlarmMessage%, Meldung
  AlarmShutdown := AlarmAction = 3 ? 1 : 0
  Gui, 4: Add, Radio, xm+200 yp vAlarmShutdown gALARMPROG Checked%AlarmShutdown%, Herunterfahren
  AlarmProg := AlarmAction = 4 ? 1 : 0
  Gui, 4: Add, Radio, xm+10 ys+50 vAlarmProg gALARMPROG Checked%AlarmProg%, Programm:
  foo := AlarmAction = 4 ? 0 : 1
  Gui, 4: Add, Edit, xm+100 yp-3 w170 r1 vAlarmExec Disabled%foo%, %AlarmExec%
  Gui, 4: Add, Button, gPROGNAME vProgBttn xm+277 yp-1 w23 Disabled%foo%, ...
  Gui, 4: Add, Button, Default xm+125 yp+50 w90, &OK
  Gui, 4: Add, Button, g4GUICLOSE x+5 W90, &Abbrechen
  Gui, 4: Show,, Alarm-Editor (%EditorTitle%)
  OnMessage(0x201, "WM_LBUTTONDOWN4")
Return

#IfWinActive, ahk_group GroupList
F2::GoSub, ALARMCHANGE
F3::GoSub, ALARMDUPL
Del::GoSub, ALARMDELETE
^Enter::GoSub, ALARMEXEC
!Enter::GoSub, PROPERTIES
^NumpadEnter::GoSub, ALARMEXEC
!NumpadEnter::GoSub, PROPERTIES
#IfWinActive, ahk_group GroupEdit
F5::GoSub, GoToday
#IfWinActive

GoToday:
  GuiControlGet, focusControl, 4: FocusV
  If (focusControl = "AlarmDate" || focusControl = "AlarmTime")
    GuiControl, 4:, %focusControl%, %A_Now%
Return
;------------------------------------------------------------------------------

PROGNAME:
  Gui, 4: +OwnDialogs ; become modal, no interact with the GUI window until dismissing the dialog.
  FileSelectFile, SelectedFile, 59, %AlarmExec%, Programm auswählen, Programme (*.exe)
  If not SelectedFile
      Return
  GuiControl, 4:, AlarmExec, %SelectedFile%
Return

ALARMPROG:
  Gui, 4: Submit, NoHide
  If (AlarmProg = 1)
   {
     GuiControl, 4: Enable, AlarmExec
     GuiControl, 4: Enable, ProgBttn
   }
  Else
   {
     GuiControl, 4: Disable, AlarmExec
     GuiControl, 4: Disable, ProgBttn
   }
Return

4ButtonOK:
  Gui, 4: +OwnDialogs ; wg. MsgBox
  Gui, 4: Submit, NoHide
  If (AlarmName = "" or AlarmName = "Geben Sie hier eine Benennung an.")
    {
      MsgBox, 48, Achtung!, Geben Sie einen Namen an!
      ControlFocus, Edit1, ahk_id %Gui4ID%
      Return
    }
  Else If (AlarmBeep = 0 && AlarmMessage = 0 && AlarmShutdown = 0 && AlarmProg = 0)
    {
      MsgBox, 48, Achtung!, Geben Sie eine Aktion an!
      ControlFocus, Button3, ahk_id %Gui4ID%
      Return
    }
  GoSub, 4GuiClose
  Gui, 3: Default ; sonst funktioniert LV_Add nicht
  FormatTime, foo, %AlarmTime%, HH:mm:ss ; Time

  If (!AlarmRow)
    LV_Add("Select Check", AlarmName, foo, Fulltext(AlarmRerun))
  Else
    LV_Modify(AlarmRow, "Select Check", AlarmName, foo, Fulltext(AlarmRerun))

  RowNumber := LV_GetNext()
  AlarmName%RowNumber% := AlarmName
  AlarmTime%RowNumber% := AlarmTime
  AlarmDate%RowNumber% := AlarmDate
  AlarmRerun%RowNumber% := AlarmRerun
  AlarmAction%RowNumber% := AlarmBeep = 1 ? 1 : AlarmMessage = 1 ? 2 : AlarmShutdown = 1 ? 3 : AlarmProg = 1 ? 4 : 0
  AlarmExec%RowNumber% := AlarmExec
Return

4GuiEscape:
4GuiClose:
  Gui, 3: -Disabled
  Gui, 4: Destroy
  ControlFocus, Button4, ahk_id %Gui3ID% ; focus to the OK button
  OnMessage(0x201, "WM_LBUTTONDOWN")
Return
;------------------------------------------------------------------------------

ALLSAVE:
  Gui, 3: Submit, NoHide
  Loop, % LV_GetCount()
    AlarmCheck%A_Index% := (LV_GetNext(A_Index - 1, "Checked") = A_Index) ? 1 : 0
  Loop, 9  ; alle Alarm-Sektionen löschen
    IniDelete, %IniPath%, % "Alarm" . A_Index
  Loop, % LV_GetCount()
    {
    IniWrite, % AlarmCheck%A_Index%,              %IniPath%, % "Alarm" . A_Index, AlarmCheck%A_Index%
    IniWrite, % AlarmName%A_Index%,               %IniPath%, % "Alarm" . A_Index, AlarmName%A_Index%
    IniWrite, % SubStr((AlarmTime%A_Index%), -5), %IniPath%, % "Alarm" . A_Index, AlarmTime%A_Index%
    IniWrite, % SubStr(AlarmDate%A_Index%, 1, 8), %IniPath%, % "Alarm" . A_Index,  AlarmDate%A_Index%
    IniWrite, % AlarmRerun%A_Index%,              %IniPath%, % "Alarm" . A_Index, AlarmRerun%A_Index%
    IniWrite, % AlarmAction%A_Index%,             %IniPath%, % "Alarm" . A_Index, AlarmAction%A_Index%
    IniWrite, % AlarmExec%A_Index%,               %IniPath%, % "Alarm" . A_Index, AlarmExec%A_Index%
    }
  GoSub, 3GUICLOSE
; execString =
  GoSub, InitializeTimers ; u.a. wird "execString" produziert
; StringReplace, execString, execString, `n ; entfernt Zeilenumbruch am Anfang
; MsgBox, 262144, Geplante Ausführungen, % RTrim(execString, OmitChars = " `n")
Return

3GUICLOSE:
3GUIESCAPE:
  Gui, 3: Destroy
Return

; _____________________________________________________________________________
; ——————————————————————————————————————————————————————————————————————————————
; ||  I N F O  //
; ——————————————
; ¯¯¯¯¯¯¯¯¯¯¯¯¯
ABOUT:
  Gui, 6: Destroy
  Gui, 6: +LastFound -Caption +Border +AlwaysOnTop
  Gui6ID := WinExist()
  Gui, 6: Font, s9 norm, MS Sans Serif
  Gui, 6: Color, cFFFFFF
  Gui, 6: Margin, 10, 10
  StringReplace, NewStr, ScriptName, &, + ; auch Chr(38) funktioniert nicht
  bitVersion := A_PtrSize = "8" ? "64bit" : "32bit"
  Gui, 6: Add, Text,, %NewStr%`, Ver. %ScriptVersion%.%ScriptBuild% (%ScriptDate%)`, %bitVersion%
  If !A_IsCompiled
    Gui, 6: Add, Text,, AutoHotkey-Skript (AutoHotkey-Version %A_AHKVersion%)
  Gui, 6: Add, Text,, (C) %A_YYYY% W. Happe, GNU General Public License
  Gui, 6: Font, underline
  Gui, 6: Add, Text, cNavy gHOMEPAGE vwebSite, %webSite%
  Gui, 6: Show,, Über %ScriptName%
  hCurs:=DllCall("LoadCursor", "UInt", NULL, "Int", 32649, "UInt") ;IDC_HAND
  OnMessage(0x200, "WM_MOUSEMOVE6")
  OnMessage(0x201, "WM_LBUTTONDOWN6")
Return
;------------------------------------------------------------------
6GUICLOSE:
6GUIESCAPE:
  OnMessage(0x200, "WM_MOUSEMOVE")
  DllCall("DestroyCursor","Uint",hCur)
  OnMessage(0x201, "WM_LBUTTONDOWN") ; for moving the clock
  DllCall("AnimateWindow","UInt",Gui6ID,"Int",500,"UInt","0x90000")
  Gui, 6: Destroy
Return

HOMEPAGE:
  Run, %webSite%
  If ErrorLevel = ERROR
    MsgBox, 48, %ScriptName%, «%webSite%» konnte nicht geöffnet werden.

Return

; _____________________________________________________________________________
; ——————————————————————————————————————————————————————————————————————————————
; ||   F u n k t i o n e n  //
; ————————————————————————————
; ¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯
DrawHands(t, ROuter, RInner, Thickness, SHColor)
  {
  Global Diameter, G, CenterX, CenterY
  R1 := Diameter//2-ceil((Diameter//2)*ROuter) ; outer position
  R2 := Diameter//2-ceil((Diameter//2)*RInner) ; inner position
  pPen := Gdip_CreatePen(0xa0 SHColor, floor((Diameter/100)*Thickness))
  Gdip_DrawLine(G, pPen
          , ceil(CenterX - (R2 * Cos(t * Atan(1) * 4 / 180)))
          , ceil(CenterY - (R2 * Sin(t * Atan(1) * 4 / 180)))
          , ceil(CenterX - (R1 * Cos(t * Atan(1) * 4 / 180)))
          , ceil(CenterY - (R1 * Sin(t * Atan(1) * 4 / 180))))
  Gdip_DeletePen(pPen)
  }

FormatNextExec(nrSeconds)
  {
  timeString := A_Now
  timeString += %nrSeconds%, Seconds
  FormatTime, timeString, %timeString%, dd.MM.yy, HH:mm:ss ; yyyyMMddHHmmss ;
  return %timeString%
  }

NextExec1(AlarmDT)
  {  ; einmalig
  DiffTime := AlarmDT
  EnvSub, DiffTime, %A_Now%, Seconds ; Minus wenn AlarmDate abgelaufen
  return %DiffTime%
  }

NextExec2(AlarmDT)
  {  ; stündlich
  NewTime := A_Now
  FormatTime, ATime, %AlarmDT%, s
  FormatTime, NTime, %NewTime%, s
  If (NTime < ATime) ;) && SubStr(NTime, 1, 2) = SubStr(ATime, 1, 2))
    NewTime += -1, Minutes ; NewTime += % SubStr(NTime, -1) * -1, Seconds
  FormatTime, AMinute, %AlarmDT%, m
  Loop, 60
    {
    NewTime += 1, Minutes
    FormatTime, NMinute, %NewTime%, m
    If (NMinute = AMinute)
      Break
    }
  DiffTime := SubStr(NewTime, 1, 10) . SubStr(AlarmDT, -3) ; -3 extract 4 last characters
  EnvSub, DiffTime, %A_Now%, Seconds ; Minus wenn AlarmDate abgelaufen
  return %DiffTime%
  }

NextExec3(AlarmDT)
  {  ; täglich
  NewTime := A_Now
  FormatTime, ATime, %AlarmDT%, mss
  FormatTime, NTime, %NewTime%, mss
  If (NTime < ATime)
    NewTime += -1, Hours ; NewTime += % (SubStr(NTime, 3, 2) * 60 + SubStr(NTime, -1)) * -1, Seconds
;  MsgBox % NTime "`n" ATime "`n`n" A_Now "`n" NewTime
  FormatTime, AHour, %AlarmDT%, H ; Hours without leading zero; 24-hour format (0 - 23)
  Loop, 24
    {
    NewTime += 1, Hours
    FormatTime, NHour, %NewTime%, H
    If (NHour = AHour)
      Break
    }
  DiffTime := SubStr(NewTime, 1, 10) . SubStr(AlarmDT, -3)
  EnvSub, DiffTime, %A_Now%, Seconds ; Minus wenn AlarmDate abgelaufen
  return %DiffTime%
  }

NextExec4(AlarmDT)
  {  ; wöchentlich - am gleichen Wochentag
  NewTime := A_Now
  FormatTime, ATime, %AlarmDT%, Hmmss
  FormatTime, NTime, %NewTime%, Hmmss
  If (NTime < ATime)
    NewTime += -1, Days
  FormatTime, AWDay, %AlarmDT%, WDay ; Day of the week (1 – 7)
  Loop, 7
    {
    NewTime += 1, Days
    FormatTime, NWDay, %NewTime%, WDay ; Day of the week (1 – 7)
    If (NWDay = AWDay)
      Break
    }
  DiffTime := SubStr(NewTime, 1, 8) . SubStr(AlarmDT, -5)
  EnvSub, DiffTime, %A_Now%, Seconds ; Minus wenn AlarmDate abgelaufen
  return %DiffTime%
  }

NextExec5(AlarmDT)
  {  ; monatlich - am glechen Tag des Monats (z.B. 3. Mittwoch oder 25. des Monats)
  NewTime := A_Now
  FormatTime, ATime, %AlarmDT%, Hmmss
  FormatTime, NTime, %NewTime%, Hmmss
  If (NTime < ATime)
    NewTime += -1, Days
  FormatTime, AWDay, %AlarmDT%, WDay ; Day of the week (1 – 7). Sunday is 1.
  FormatTime, AMDay, %AlarmDT%, d   ; Day of the month (01 – 31)
  DayOfMonth := Ceil(AMDay / 7)
  NDayOfM := Ceil(A_MDay / 7)
  Loop, 31
    {
    NewTime += 1, days
    FormatTime, NWDay, %NewTime%, WDay ; Day of the week (1 – 7). Sunday is 1.
    FormatTime, NMDay, %NewTime%, d   ; Day of the month (01 – 31)
    If ((PrefDay && AWDay = NWDay && DayOfMonth = NDayOfM) || (!PrefDay && AMDay = NMDay))
      Break
    }
  DiffTime := SubStr(NewTime, 1, 8) . SubStr(AlarmDT, -5)
  EnvSub, DiffTime, %A_Now%, Seconds ; Minus wenn AlarmDate abgelaufen
  return %DiffTime%
  }

NextExec6(AlarmDT)
  {  ; alle 3 Monate - s.o.
  NewTime := A_Now
  FormatTime, ATime, %AlarmDT%, Hmmss
  FormatTime, NTime, %NewTime%, Hmmss
  If (NTime < ATime)
    NewTime += -1, Days
  FormatTime, AWDay, %AlarmDateDT%, WDay ; Day of the week (1 – 7). Sunday is 1.
  FormatTime, AMDay, %AlarmDateDT%, d   ; Day of the month (01 – 31)
  FormatTime, AMonth, %AlarmDateDT%, M  ; Month with leading zero (01 – 12)
  DayOfMonth := Ceil(AMDay / 7)
  NDayOfM := Ceil(A_MDay / 7)
  AQrt := RegExMatch(AMonth, "12|09|06|03") ? 3 : RegExMatch(AMonth, "11|08|05|02") ? 2 : 1
  AMon := RegExMatch(A_MM, "12|09|06|03") ? 3 : RegExMatch(A_MM, "11|08|05|02") ? 2 : 1
  Loop, 92
    {
    NewTime += 1, days
    FormatTime, NWDay, %NewTime%, WDay ; Day of the week (1 – 7). Sunday is 1.
    FormatTime, NMDay, %NewTime%, d   ; Day of the month (01 – 31)
    If ((AQrt = AMon) && ((PrefDay && AWDay = NWDay && DayOfMonth = NDayOfM) || (!PrefDay && AMDay = NMDay)))
      Break
    }
  DiffTime := SubStr(NewTime, 1, 8) . SubStr(AlarmDT, -5)
  EnvSub, DiffTime, %A_Now%, Seconds ; Minus wenn AlarmDate abgelaufen
  return %DiffTime%
  }

NextExec7(AlarmDT)
  {  ; jährlich - s.o.
  NewTime := A_Now
  FormatTime, ATime, %AlarmDT%, Hmmss
  FormatTime, NTime, %NewTime%, Hmmss
  If (NTime < ATime)
    NewTime += -1, Days
  FormatTime, AWDay, %AlarmDateDT%, WDay ; Day of the week (1 – 7). Sunday is 1.
  FormatTime, AMDay, %AlarmDateDT%, d   ; Day of the month (1 – 31)
  FormatTime, AMonth, %AlarmDateDT%, M  ; Month (1 – 12)
  DayOfMonth := Ceil(AMDay / 7)
  NDayOfM := Ceil(A_MDay / 7)
  Loop, 365
    {
    NewTime += 1, days
    FormatTime, NWDay, %NewTime%, WDay ; Day of the week (1 – 7). Sunday is 1.
    FormatTime, NMDay, %NewTime%, d   ; Day of the month (1 – 31)
    FormatTime, NMonth, %NewTime%, M  ; Month (1 – 12)
    If (AMonth = NMonth && ((PrefDay && AWDay = NWDay && DayOfMonth = NDayOfM) || (!PrefDay && AMDay = NMDay)))
      Break
    }
  DiffTime := SubStr(NewTime, 1, 8) . SubStr(AlarmDT, -5)
  EnvSub, DiffTime, %A_Now%, Seconds ; Minus wenn AlarmDate abgelaufen
  return %DiffTime%
  }

ResetTimer(AIndex) ; AIndex liegt zwischen 1 bis max. 9 (Aufgabenliste)
  {
  ExecNum := AlarmRerun%AIndex%
  ExecTime := ActionTime%AIndex% ; TimeStamp der letzten Ausführung
  If (ExecTime > 0 && ExecNum <> 1)
    { ; mehrfache Starts innerhalb einer Sekunde verhindern
    EnvSub, ExecTime, %A_Now%, Seconds
    IfEqual, ExecTime, 0, Return
    }
  AlarmDT := SubStr(AlarmDate%AIndex%, 1, 8) . SubStr((AlarmTime%AIndex%), -5)
  NewTime := A_Now
  DiffTime := NextExec%ExecNum%(AlarmDT) ; DiffTime immer in Sekunden!
  If (ExecNum = 1) ; einmalige Ausführung
    {
    If (DiffTime > 1)
      SetTimer, AlarmTimer%AIndex%, % DiffTime * 1000
    Else ; - Timer nicht neu starten, Aufgabe löschen!
      {
      AlarmCheck%AIndex% = ; = 0 um Aufgaben nur zu deaktivieren
      AlarmName%AIndex% =
      AlarmTime%AIndex% =
      AlarmDate%AIndex% =
      AlarmRerun%AIndex% =
      AlarmAction%AIndex% =
      AlarmExec%AIndex% =
      }
    } ; Ende: einmalige Ausführung gewählt
  Else If (ExecNum = 2 && DiffTime > 1) ; stündlich
    SetTimer, AlarmTimer%AIndex%, % NextExec2(AlarmDT) * 1000
  Else If (ExecNum = 3 && DiffTime > 1) ; täglich
    SetTimer, AlarmTimer%AIndex%, % NextExec3(AlarmDT) * 1000
  Else If (ExecNum = 4 && DiffTime > 1) ; wöchentlich - am gleichen Wochentag
    SetTimer, AlarmTimer%AIndex%, % NextExec4(AlarmDT) * 1000
  Else If (ExecNum = 5 && DiffTime > 1) ; monatlich - am glechen Tag des Monats (z.B. 3. Mittwoch oder 25. des Monats)
    SetTimer, AlarmTimer%AIndex%, % NextExec5(AlarmDT) * 1000
  Else If (ExecNum = 6 && DiffTime > 1) ; alle 3 Monate - s.o.
    SetTimer, AlarmTimer%AIndex%, % NextExec6(AlarmDT) * 1000
  Else If (ExecNum = 7 && DiffTime > 1) ; jährlich - s.o.
    SetTimer, AlarmTimer%AIndex%, % NextExec7(AlarmDT) * 1000
  }

ACTION_1(AName, ARerun, AExec, AReText) ; Ton
  {
  Duration = 0
  SetTimer, BEEPTimer, 500
  MsgBox, 262144, Alarm-Liste, % AName "`n`nDiese Meldung erscheint " AReText "."
  SetTimer, BEEPTimer, Off
Return
BEEPTimer: ; nicht in die Klammern der Funktion packen, sonst wird Beep durchlaufen!
  SoundPlay *48 ; Exclamation
  Sleep, 250
  SoundPlay *64 ;  Asterisk (info)
  Duration := Duration + 0.5
  If Duration = % BeepStop * 60
    SetTimer, BEEPTimer, Off
Return
}

ACTION_2(AName, ARerun, AExec, AReText) ; Meldung
  {
  SoundPlay *-1  ; Simple beep. If the sound card is not available, generated using the speaker.
  Gui, 5: Destroy
  Gui, 5: +LastFound +AlwaysOnTop -MinimizeBox -MaximizeBox
  Gui5ID := WinExist()
  Gui, 5: Margin, 10, 10
  Gui, 5: Add, Progress, BackgroundFFFFFF x0 y-1 w218 h110
  FontSize := StrLen(AName) < 20 ? 20 : StrLen(AName) < 40 ? 16 : StrLen(AName) < 60 ? 12 : 10
  Gui, 5: Font, s%FontSize% bold cRed, MS Sans Serif ; Limit80
  Gui, 5: Add, Text, xm ym+10 w200 center BackgroundTrans, %AName%
  Gui, 5: Font, s9 norm cDefault
  Gui, 5: Add, Text, xm yp+65 w200 center BackgroundTrans, Diese Meldung erscheint %AReText%.
  Gui, 5: Add, Button, Default xm+63 y120 w75, &OK
  Gui, 5: Show, w220, Alarm-Liste
  WinWaitClose, ahk_id %Gui5ID%,, 1 ; damit das Fenster aktiv ist, 1 s reicht dafür
Return
5GUICLOSE:
5GUIESCAPE:
5BUTTONOK:
  Gui, 5: Destroy
Return
}

ACTION_3(AName, ARerun, AExec, AReText) ; Herunterfahren
  {
  global Reboot, static CountDown := 60
  FormatTime, ADate, %A_Now%, LongDate
  FormatTime, ATime, %A_Now%, HH:mm:ss
  bar := ARerun = 1 ? " (Alarm wird gelöscht)" : ""
  baz := Reboot ? "`nAktion:`tNEUSTART!" : "`nAktion:`tHerunterfahren"
  TxtShutdown := "Name: `t" AName "`nUhrzeit: `t" ATime "`nDatum: `t" ADate "`nAusf.: `t" AReText bar baz
  IconDLL := A_WinDir "\System32\User32.dll"
  Gui, 7: Destroy
  Gui, 7: +Owner1 ; no taskbar button, always on top of its owner
  Gui, 1: +Disabled
  Gui, 7: +LastFound +AlwaysOnTop -MinimizeBox -MaximizeBox -SysMenu
  Gui7ID := WinExist()
  Gui, 7: Add, Pic, x10 y20 Icon4, %IconDLL%
  Gui, 2: font, norm s9, MS Sans Serif
  Gui, 7: Margin, 50, 10
  Gui, 7: Add, Text, xm ym+10 w165, Das System wird heruntergefahren.`nSpeichern Sie alle Daten.`nAlle Änderungen, die nicht gespei-`nchert werden, gehen verloren.
  Gui, 7: Add, Text, xm yp+65, Zeit bis zum Herunterfahren:
  Gui, 7: Add, Text, xm+140 yp w40 vCountDown, 60 s
  Gui, 7: Add, GroupBox, x10 ym+100 Section w210 h95, Alarm
  Gui, 7: Add, Text, xs+10 ys+20, %TxtShutdown%
  Gui, 7: Show, w230 h215, System herunterfahren
  Loop, 60
    {
    Sleep, 1000
    CountDown -= 1
    SoundPlay, *48 ; Exclamation. If the sound card is not available, the sound is generated using the speaker.
    GuiControl, 7: Text, CountDown, %CountDown% s
    }
  ParShutdown := Reboot = 1 ? 6 : 5
  Shutdown, %ParShutdown% ; The "Force" value (4) forces all open applications to close
  GoSub, PROGEXIT
  }

ACTION_4(AName, ARerun, AExec, AReText) ; Programm ausführen
  {
  global NoConfirm
  If (!NoConfirm)
    {
    MsgBox, 262209, %ScriptName%, % "Bitte bestätigen Sie die Ausführung:`n`n" AName "`n`n" AExec
    IfMsgBox, Cancel
      Return
    }
  If (A_IsAdmin || RegExMatch(A_OSVersion, "^[WIN_NT4|WIN_95|WIN_98|WIN_ME|WIN_2000|WIN_XP|WIN_2003]+$")) ; not WIN_VISTA
    Run, %AExec%,, UseErrorLevel
  Else
    Run, *RunAs %AExec%,, UseErrorLevel
  If ErrorLevel = ERROR
      MsgBox, 48, %ScriptName%, «%AExec%» konnte nicht gestartet werden.
  }

Fulltext(x)
  {
  return x = 1 ? "Einmal" : x = 2 ? "Stündlich" : x = 3 ? "Täglich" : x = 4 ? "Wöchentlich" : x = 5 ? "Monatlich" : x = 6 ? "Alle 3 Monate" : "Jährlich"
  }

Shorttext(x)
  {
  return x = 1 ? "Einmal" : x = 2 ? "Stündl." : x = 3 ? "Täglich" : x = 4 ? "Wchntl." : x = 5 ? "Monatl." : x = 6 ? "Alle 3 M." : "Jährlich"
  }

WM_MOUSEMOVE6()
  {
  global hCurs, webSite
  If (A_GuiControl = "webSite")
    DllCall("SetCursor", "UInt", hCurs)
  }

WM_LBUTTONDOWN4()
  {
    Global AlarmName
;   IfNotEqual, A_Gui, 4, Return
    If (A_GuiControl = "AlarmName")
      {
      GuiControlGet, AlarmName, 4:, AlarmName
      If (AlarmName = "Geben Sie hier eine Benennung an.")
        GuiControl,, AlarmName, ; nichts
     }
  }

WM_LBUTTONDOWN6()
  {
;   IfNotEqual, A_Gui, 6, Return
    IfNotEqual, A_GuiControl, webSite, GoSub, 6GUICLOSE
  }


WM_LBUTTONDOWN() {
  Global hCurs ;, Gui1ID ; xPos, yPos, Gui1ID,
  IfNotEqual, A_Gui, 1, Return ; following actions only in this GUI
  SetTimer, REMOVETT, -10 ; ToolTip
    SetTimer, SHOWTIME, Off
    If GetKeyState("LButton", "P")
    {
      DllCall("SetCursor", "UInt", hCurs)
      PostMessage, 0xA1, 2 ; posting WM_NCLBUTTONDOWN message (0xA1) to HTCAPTION (2)
    }
  }


WM_LBUTTONDBLCLK() {
  If (A_Gui = 1)
    {
    GoSub, ALARMS
    SetTimer, SHOWTIME, Off
    }
  }


; WM_MOUSEMOVE(wParam, lParam, Msg, hwnd) {
WM_MOUSEMOVE() {
    Global  ; tThWnd1, Gui1ID, hdc, ClickThrough, xPos, yPos, _Width, _Height, Trans ;, A_TT
    ;  IfNotEqual, A_Gui, 1, Return ; following actions only in this GUI
    If (A_Gui = 1 and not TransTemp)
      ; If (hwnd = Gui1ID and not TransTemp)
      {
        If (ClickThrough)
          {
            WinGetPos, xPos, yPos,,, ahk_id %Gui1ID%
            TransTemp := GetKeyState("LControl", "P") ? 255 : 8
            UpdateLayeredWindow(Gui1ID, hdc, "", "", "", "", TransTemp)
            WinSet, ExStyle, +0x20, ahk_id %Gui1ID%
            SetTimer, CLICKTHROUGH, 100
          }
        Else If !WinExist("ahk_id " . tThWnd1)
            SetTimer, SHOWTIME, -1000
      }
  }

CLICKTHROUGH:
  CoordMode, Mouse, Screen
  MouseGetPos, VarX, VarY ;,,, 0
  If !((VarX >= xPos and VarX <= (xPos + iWidth)) And (VarY >= yPos and VarY <= (yPos + iHeight)))
    {
 ;    ToolTip, % VarX "`n" VarY "`n" xPos "`n"  yPos
      SetTimer, CLICKTHROUGH, Off
      WinSet, ExStyle, -0x20, ahk_id %Gui1ID%
      UpdateLayeredWindow(Gui1ID, hdc, "", "", "", "", TransNorm)
      TransTemp =
    }
Return

AHK_NOTIFYICON(wParam, lParam)
  {
  FormatTime, TimeStr,, ddd'.,' d'.' MMM'.' yyyy', 'H:mm
  Menu, Tray, Tip, %TimeStr%
  }

; ##############################################################################
; Gdip standard library v1.44 by tic (Tariq Porter) 22/08/11
; Modified by fincs (30/08/11) to support the 64-bit build of AutoHotkey
;#####################################################################################

UpdateLayeredWindow(hwnd, hdc, x="", y="", w="", h="", Alpha=255)
{
	if ((x != "") && (y != ""))
		VarSetCapacity(pt, 8), NumPut(x, pt, 0, "uint"), NumPut(y, pt, 4, "uint")

	if (w = "") ||(h = "")
		WinGetPos,,, w, h, ahk_id %hwnd%

	return DllCall("UpdateLayeredWindow", "ptr", hwnd, "ptr", 0, "ptr", ((x = "") && (y = "")) ? 0 : &pt
	, "int64*", w|h<<32, "ptr", hdc, "int64*", 0, "uint", 0, "uint*", Alpha<<16|1<<24, "uint", 2)
}

BitBlt(ddc, dx, dy, dw, dh, sdc, sx, sy, Raster="")
{
	return DllCall("gdi32\BitBlt", "ptr", dDC, "int", dx, "int", dy, "int", dw, "int", dh
	, "ptr", sDC, "int", sx, "int", sy, "uint", Raster ? Raster : 0x00CC0020)
}

SetImage(hwnd, hBitmap)
{
	SendMessage, 0x172, 0x0, hBitmap,, ahk_id %hwnd%
	E := ErrorLevel
	DeleteObject(E)
	return E
}

CreateRectF(ByRef RectF, x, y, w, h)
{
   VarSetCapacity(RectF, 16)
   NumPut(x, RectF, 0, "float"), NumPut(y, RectF, 4, "float"), NumPut(w, RectF, 8, "float"), NumPut(h, RectF, 12, "float")
}


CreateRect(ByRef Rect, x, y, w, h)
{
	VarSetCapacity(Rect, 16)
	NumPut(x, Rect, 0, "uint"), NumPut(y, Rect, 4, "uint"), NumPut(w, Rect, 8, "uint"), NumPut(h, Rect, 12, "uint")
}


CreatePointF(ByRef PointF, x, y)
{
   VarSetCapacity(PointF, 8)
   NumPut(x, PointF, 0, "float"), NumPut(y, PointF, 4, "float")
}

CreateDIBSection(w, h, hdc="", bpp=32, ByRef ppvBits=0)
{
	hdc2 := hdc ? hdc : GetDC()
	VarSetCapacity(bi, 40, 0)
	NumPut(w, bi, 4, "uint"), NumPut(h, bi, 8, "uint"), NumPut(40, bi, 0, "uint"), NumPut(1, bi, 12, "ushort"), NumPut(0, bi, 16), NumPut(bpp, bi, 14, "ushort")
	hbm := DllCall("CreateDIBSection", "ptr" , hdc2, "ptr" , &bi, "uint" , 0, "ptr*", ppvBits, "ptr" , 0, "uint" , 0, "ptr")

	if !hdc
		ReleaseDC(hdc2)
	return hbm
}

DestroyIcon(hIcon)
{
   return DllCall("DestroyIcon", "ptr", hIcon)
}

CreateCompatibleBitmap(hdc, w, h)
{
	return DllCall("gdi32\CreateCompatibleBitmap", "ptr", hdc, "int", w, "int", h, "ptr")
}

CreateCompatibleDC(hdc=0)
{
   return DllCall("CreateCompatibleDC", "ptr", hdc, "ptr")
}


SelectObject(hdc, hgdiobj)
{
   return DllCall("SelectObject", "ptr", hdc, "ptr", hgdiobj, "ptr")
}

DeleteObject(hObject)
{
   return DllCall("DeleteObject", "ptr", hObject)
}

GetDC(hwnd=0)
{
	return DllCall("GetDC", "ptr", hwnd, "ptr")
}

ReleaseDC(hdc, hwnd=0)
{
   return DllCall("ReleaseDC", "ptr", hwnd, "ptr", hdc)
}

DeleteDC(hdc)
{
   return DllCall("DeleteDC", "ptr", hdc)
}

Gdip_DrawRectangle(pGraphics, pPen, x, y, w, h)
{
   return DllCall("gdiplus\GdipDrawRectangle", "ptr", pGraphics, "ptr", pPen, "float", x, "float", y, "float", w, "float", h)
}

Gdip_DrawLine(pGraphics, pPen, x1, y1, x2, y2)
{
   return DllCall("gdiplus\GdipDrawLine", "ptr", pGraphics, "ptr", pPen
   , "float", x1, "float", y1, "float", x2, "float", y2)
}

Gdip_DrawImage(pGraphics, pBitmap, dx="", dy="", dw="", dh="", sx="", sy="", sw="", sh="", Matrix=1)
{
	if (Matrix&1 = "")
		ImageAttr := Gdip_SetImageAttributesColorMatrix(Matrix)
	else if (Matrix != 1)
		ImageAttr := Gdip_SetImageAttributesColorMatrix("1|0|0|0|0|0|1|0|0|0|0|0|1|0|0|0|0|0|" Matrix "|0|0|0|0|0|1")
	if (sx = "" && sy = "" && sw = "" && sh = "")
	{
		if (dx = "" && dy = "" && dw = "" && dh = "")
		{
			sx := dx := 0, sy := dy := 0
			sw := dw := Gdip_GetImageWidth(pBitmap)
			sh := dh := Gdip_GetImageHeight(pBitmap)
		}
		else
		{
			sx := sy := 0
			sw := Gdip_GetImageWidth(pBitmap)
			sh := Gdip_GetImageHeight(pBitmap)
		}
	}
	E := DllCall("gdiplus\GdipDrawImageRectRect", "ptr", pGraphics, "ptr", pBitmap
	, "float", dx, "float", dy, "float", dw, "float", dh
	, "float", sx, "float", sy, "float", sw, "float", sh
	, "int", 2, "ptr", ImageAttr, "ptr", 0, "ptr", 0)
	if ImageAttr
		Gdip_DisposeImageAttributes(ImageAttr)
	return E
}

Gdip_SetImageAttributesColorMatrix(Matrix)
{
	VarSetCapacity(ColourMatrix, 100, 0)
	Matrix := RegExReplace(RegExReplace(Matrix, "^[^\d-\.]+([\d\.])", "$1", "", 1), "[^\d-\.]+", "|")
	StringSplit, Matrix, Matrix, |
	Loop, 25
	{
		Matrix := (Matrix%A_Index% != "") ? Matrix%A_Index% : Mod(A_Index-1, 6) ? 0 : 1
		NumPut(Matrix, ColourMatrix, (A_Index-1)*4, "float")
	}
	DllCall("gdiplus\GdipCreateImageAttributes", "ptr*", ImageAttr)
	DllCall("gdiplus\GdipSetImageAttributesColorMatrix", "ptr", ImageAttr, "int", 1, "int", 1, "ptr", &ColourMatrix, "ptr", 0, "int", 0)
	return ImageAttr
}

Gdip_GraphicsFromImage(pBitmap)
{
    DllCall("gdiplus\GdipGetImageGraphicsContext", "ptr", pBitmap, "ptr*", pGraphics)
    return pGraphics
}

Gdip_GraphicsFromHDC(hdc)
{
    DllCall("gdiplus\GdipCreateFromHDC", "ptr", hdc, "ptr*", pGraphics)
    return pGraphics
}

Gdip_GetImageWidth(pBitmap)
{
   DllCall("gdiplus\GdipGetImageWidth", "ptr", pBitmap, "uint*", Width)
   return Width
}

Gdip_GetImageHeight(pBitmap)
{
   DllCall("gdiplus\GdipGetImageHeight", "ptr", pBitmap, "uint*", Height)
   return Height
}

Gdip_GetImageDimensions(pBitmap, ByRef Width, ByRef Height)
{
	DllCall("gdiplus\GdipGetImageWidth", "ptr", pBitmap, "uint*", Width)
	DllCall("gdiplus\GdipGetImageHeight", "ptr", pBitmap, "uint*", Height)
}

Gdip_CreateBitmapFromFile(sFile, IconNumber=1, IconSize="")
{
	SplitPath, sFile,,, ext
	if ext in exe,dll
	{
		Sizes := IconSize ? IconSize : 256 "|" 128 "|" 64 "|" 48 "|" 32 "|" 16
		VarSetCapacity(buf, 32+2*A_PtrSize)
		Loop, Parse, Sizes, |
		{
			DllCall("PrivateExtractIcons", "str", sFile, "int", IconNumber-1, "int", A_LoopField, "int", A_LoopField, "ptr*", hIcon, "ptr*", 0, "uint", 1, "uint", 0)
			if !hIcon
				continue
			if !DllCall("GetIconInfo", "ptr", hIcon, "ptr", &buf)
			{
				DestroyIcon(hIcon)
				continue
			}
			hbmColor := NumGet(buf, 12+A_PtrSize)
			hbmMask  := NumGet(buf, 12)
			if !(hbmColor && DllCall("GetObject", "ptr", hbmColor, "int", 24, "ptr", &buf))
			{
				DestroyIcon(hIcon)
				continue
			}
			break
		}
		if !hIcon
			return -1
		Width := NumGet(buf, 4, "int"),  Height := NumGet(buf, 8, "int")
		hbm := CreateDIBSection(Width, -Height), hdc := CreateCompatibleDC(), obm := SelectObject(hdc, hbm)
		if !DllCall("DrawIconEx", "ptr", hdc, "int", 0, "int", 0, "ptr", hIcon, "uint", Width, "uint", Height, "uint", 0, "ptr", 0, "uint", 3)
		{
			DestroyIcon(hIcon)
			return -2
		}
		VarSetCapacity(dib, 84)
		DllCall("GetObject", "ptr", hbm, "int", A_PtrSize = 8 ? 96 : 84, "ptr", &dib) ; sizeof(DIBSECTION) = 76+2*(A_PtrSize=8?4:0)+2*A_PtrSize
		Stride := NumGet(dib, 12, "int"), Bits := NumGet(dib, 20+(A_PtrSize=8?4:0)) ; padding
		DllCall("gdiplus\GdipCreateBitmapFromScan0", "int", Width, "int", Height, "int", Stride, "int", 0x26200A, "ptr", Bits, "ptr*", pBitmapOld)
		pBitmap := Gdip_CreateBitmap(Width, Height), G := Gdip_GraphicsFromImage(pBitmap)
		Gdip_DrawImage(G, pBitmapOld, 0, 0, Width, Height, 0, 0, Width, Height)
		SelectObject(hdc, obm), DeleteObject(hbm), DeleteDC(hdc)
		Gdip_DeleteGraphics(G), Gdip_DisposeImage(pBitmapOld)
		DestroyIcon(hIcon)
	}
	else
		DllCall("gdiplus\GdipCreateBitmapFromFile", "wstr", sFile, "ptr*", pBitmap)
	return pBitmap
}

Gdip_CreateBitmapFromHBITMAP(hBitmap, Palette=0)
{
	DllCall("gdiplus\GdipCreateBitmapFromHBITMAP", "ptr", hBitmap, "ptr", Palette, "ptr*", pBitmap)
	return pBitmap
}

Gdip_CreateHBITMAPFromBitmap(pBitmap, Background=0xffffffff)
{
	DllCall("gdiplus\GdipCreateHBITMAPFromBitmap", "ptr", pBitmap, "ptr*", hbm, "uint", Background)
	return hbm
}

Gdip_CreateBitmap(Width, Height, Format=0x26200A)
{
    DllCall("gdiplus\GdipCreateBitmapFromScan0", "int", Width, "int", Height, "int", 0, "int", Format, "ptr", 0, "ptr*", pBitmap)
    Return pBitmap
}

Gdip_CreatePen(ARGB, w)
{
   DllCall("gdiplus\GdipCreatePen1", "uint", ARGB, "float", w, "int", 2, "ptr*", pPen)
   return pPen
}

Gdip_BrushCreateSolid(ARGB=0xff000000)
{
	DllCall("gdiplus\GdipCreateSolidFill", "uint", ARGB, "ptr*", pBrush)
	return pBrush
}

Gdip_CloneBrush(pBrush)
{
	DllCall("gdiplus\GdipCloneBrush", "ptr", pBrush, "ptr*", pBrushClone)
	return pBrushClone
}

Gdip_DeletePen(pPen)
{
   return DllCall("gdiplus\GdipDeletePen", "ptr", pPen)
}

Gdip_DeleteBrush(pBrush)
{
   return DllCall("gdiplus\GdipDeleteBrush", "ptr", pBrush)
}

Gdip_DisposeImage(pBitmap)
{
   return DllCall("gdiplus\GdipDisposeImage", "ptr", pBitmap)
}

Gdip_DeleteGraphics(pGraphics)
{
   return DllCall("gdiplus\GdipDeleteGraphics", "ptr", pGraphics)
}

Gdip_DisposeImageAttributes(ImageAttr)
{
	return DllCall("gdiplus\GdipDisposeImageAttributes", "ptr", ImageAttr)
}

Gdip_DeleteFont(hFont)
{
   return DllCall("gdiplus\GdipDeleteFont", "ptr", hFont)
}

Gdip_DeleteStringFormat(hFormat)
{
   return DllCall("gdiplus\GdipDeleteStringFormat", "ptr", hFormat)
}

Gdip_DeleteFontFamily(hFamily)
{
   return DllCall("gdiplus\GdipDeleteFontFamily", "ptr", hFamily)
}

Gdip_SetSmoothingMode(pGraphics, SmoothingMode)
{
   return DllCall("gdiplus\GdipSetSmoothingMode", "ptr", pGraphics, "int", SmoothingMode)
}

Gdip_SetCompositingMode(pGraphics, CompositingMode=0)
{
   return DllCall("gdiplus\GdipSetCompositingMode", "ptr", pGraphics, "int", CompositingMode)
}

Gdip_Startup()
{
	if !DllCall("GetModuleHandle", "str", "gdiplus", "ptr")
		DllCall("LoadLibrary", "str", "gdiplus")
	VarSetCapacity(si, A_PtrSize = 8 ? 24 : 16, 0), si := Chr(1)
	DllCall("gdiplus\GdiplusStartup", "uptr*", pToken, "ptr", &si, "ptr", 0)
	return pToken
}

Gdip_Shutdown(pToken)
{
	DllCall("gdiplus\GdiplusShutdown", "uptr", pToken)
	if hModule := DllCall("GetModuleHandle", "str", "gdiplus", "ptr")
		DllCall("FreeLibrary", "ptr", hModule)
	return 0
}

Gdip_LockBits(pBitmap, x, y, w, h, ByRef Stride, ByRef Scan0, ByRef BitmapData, LockMode = 3, PixelFormat = 0x26200a)
{
	CreateRect(Rect, x, y, w, h)
	VarSetCapacity(BitmapData, 16+2*A_PtrSize, 0)
	E := DllCall("Gdiplus\GdipBitmapLockBits", "ptr", pBitmap, "ptr", &Rect, "uint", LockMode, "int", PixelFormat, "ptr", &BitmapData)
	Stride := NumGet(BitmapData, 8, "int")
	Scan0 := NumGet(BitmapData, 16, "ptr")
	return E
}

Gdip_UnlockBits(pBitmap, ByRef BitmapData)
{
	return DllCall("Gdiplus\GdipBitmapUnlockBits", "ptr", pBitmap, "ptr", &BitmapData)
}
